/*
 * arch/arm/mach-cxd4132/speed.c
 *
 * Speed notify
 *
 * Copyright 2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include <linux/init.h>
#include <linux/irqflags.h>
#include <mach/hardware.h>
#include <mach/localtimer.h>
#include <mach/pm.h>
#include <mach/cxd4132_clock.h>
#include <mach/regs-clk.h>
#include <asm/io.h>

void cxd41xx_speed_notify(void)
{
#ifdef CONFIG_EJ_USE_TICK_TIMER
	unsigned long flags;
	unsigned int stat, speed;

	local_irq_save(flags);

	/* new speed */
	stat = readl(VA_FCS(FCS_CK_STATUS));
	if (stat & FCS_BUSY) {
		printk(KERN_ERR "cxd41xx_speed_notify: FCS_BUSY\n");
		goto out;
	}
	speed = ((stat & FCS_AVBHOST_STAT) == FCS_HALF) ? \
		CXD4132_SPEED_LO : CXD4132_SPEED_HI;

	/* set new speed */
	cxd4132_set_speed(speed);

	/* Adjust localtimers */
	twd_calibrate_rate_all(cxd4132_localtimer_rate());

 out:
	local_irq_restore(flags);
#endif /* CONFIG_EJ_USE_TICK_TIMER */
}
